﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Amortize {

    public class Amort {

        public decimal Principle { get; private set; }
        public decimal Interest { get; private set; }
        public int Term { get; private set; }
        public DateTime FirstPayment { get; private set; }

        public List<AmortPayment> Schedule { get; private set; }

        private decimal _payment;
        private decimal _totalInt;

        public decimal Payment {
            get { return _payment; }
        }

        public decimal TotalInterest {
            get { return _totalInt; }
        }

        public Amort(decimal principle, decimal interest, int term, DateTime firstPayment) {

            Principle = principle;
            Interest = interest;
            Term = term;
            FirstPayment = firstPayment;

            _payment = CalcPayment();

            GenerateSchedule();
        }

        /// <summary>
        /// calculate a payment
        /// </summary>
        /// <returns></returns>
        private decimal CalcPayment() {

            double mr = ((double)Interest / 100) / 12;

            if (mr < 0.0001) {
                double pmt = (double)Principle / (double)Term;
                return (decimal)(Math.Floor(pmt * 100) / 100);
            }

            return (decimal)((((double)Principle) * mr) / (1 - Math.Pow((1 + mr), ((double)Term * -1))));
        }

        /// <summary>
        /// generate an amortization schedule
        /// </summary>
        public void GenerateSchedule() {

            decimal pp = Principle;
            decimal rt = (Interest / 100) / 12;
            DateTime ppd = FirstPayment;
            decimal pint, pprn;
            decimal pmt = Math.Round(Payment, 2);

            Schedule = new List<AmortPayment>();
            _totalInt = 0;

            for (int i = 1; i <= Term; i++) {

                AmortPayment p = new AmortPayment();
                p.Number = i;
                p.Date = ppd;
                pint = pp * rt;
                pprn = pmt - pint;
                pp -= pprn;
                if (pp < 0) {
                    pp = 0;
                }
                // if last payment, add balance left
                if (i == Term) {
                    pprn += pp;
                    pp = 0;
                }
                p.Interest = pint;
                p.Principle = pprn;
                p.Balance = pp;
                Schedule.Add(p);
                _totalInt += pint;
                ppd = ppd.AddMonths(1);
            }
        }
    }

    public class AmortPayment {

        public int Number { get; set; }
        public DateTime Date { get; set; }
        public decimal Interest { get; set; }
        public decimal Principle { get; set; }
        public decimal Balance { get; set; }
    }

}
