﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Amortize {
    
    public partial class MainForm : Form {

        Amort _amort;

        public MainForm() {

            InitializeComponent();
        }

        private void Main_Load(object sender, EventArgs e) {

            txtAmount.Text = "0.00";
            txtInterest.Text = "0.00";
            txtTerm.Text = "0";
        }

        private void btnCalc_Click(object sender, EventArgs e) {

            if (dgvAmort.Columns.Count == 0) {
                SetupAmortGrid();
            }

            if (txtTerm.IntValue > 0) {

                _amort = new Amort(
                    txtAmount.DecimalValue, 
                    txtInterest.DecimalValue, 
                    txtTerm.IntValue, 
                    dtpFirstPayDate.Value);
                dgvAmort.DataSource = _amort.Schedule;

                txtPayment.Visible = true;
                lblPayment.Visible = true;
                txtPayment.Text = _amort.Payment.ToString("F2");

                lblTotalInterest.Visible = true;
                txtTotalInterest.Visible = true;
                txtTotalInterest.Text = _amort.TotalInterest.ToString("F2");

                btnPrint.Visible = true;
                printToolStripMenuItem.Enabled = true;
                printPreviewToolStripMenuItem.Enabled = true;
            }
        }

        /// <summary>
        /// setup shipment grid
        /// </summary>
        private void SetupAmortGrid() {

            dgvAmort.ReadOnly = true;
            dgvAmort.AutoGenerateColumns = false;

            dgvAmort.AllowUserToAddRows = false;
            dgvAmort.AllowUserToDeleteRows = false;
            dgvAmort.AllowUserToOrderColumns = false;
            dgvAmort.AllowUserToResizeColumns = true;
            dgvAmort.AllowUserToResizeRows = false;

            dgvAmort.MultiSelect = false;
            dgvAmort.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            dgvAmort.RowHeadersVisible = false;

            dgvAmort.BackgroundColor = Color.White;
            dgvAmort.RowsDefaultCellStyle.BackColor = Color.White;
            dgvAmort.AlternatingRowsDefaultCellStyle.BackColor = Color.LightGray;

            dgvAmort.RowHeadersVisible = false;

            DataGridViewColumn column = new DataGridViewTextBoxColumn();
            column.Name = "Payment";
            column.DataPropertyName = "Number";
            column.Width = 50;
            column.ReadOnly = true;
            dgvAmort.Columns.Add(column);

            column = new DataGridViewTextBoxColumn();
            column.Name = "Date";
            column.DataPropertyName = "Date";
            column.DefaultCellStyle.Format = "MM/dd/yyyy";
            column.Width = 50;
            column.ReadOnly = true;
            dgvAmort.Columns.Add(column);

            column = new DataGridViewTextBoxColumn();
            column.Name = "Interest";
            column.DataPropertyName = "Interest";
            column.DefaultCellStyle.Format = "F2";
            column.Width = 50;
            column.ReadOnly = true;
            dgvAmort.Columns.Add(column);

            column = new DataGridViewTextBoxColumn();
            column.Name = "Principle";
            column.DataPropertyName = "Principle";
            column.DefaultCellStyle.Format = "F2";
            column.Width = 50;
            column.ReadOnly = true;
            dgvAmort.Columns.Add(column);

            column = new DataGridViewTextBoxColumn();
            column.Name = "Balance";
            column.DataPropertyName = "Balance";
            column.DefaultCellStyle.Format = "F2";
            column.Width = 50;
            column.ReadOnly = true;
            dgvAmort.Columns.Add(column);

            dgvAmort.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
        }

        private void dtpFirstPayDate_KeyPress(object sender, KeyPressEventArgs e) {

            if (e.KeyChar == (char)Keys.Return) {
                e.Handled = true;
                SendKeys.Send("{TAB}");
            }
        }

        private void btnPrint_Click(object sender, EventArgs e) {

            using (Report report = new Report()) {
                report.Amort = _amort;
                report.Print();
            }
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e) {

            // Create the print dialog object and set options
            using (PrintDialog d = new PrintDialog()) {
                d.AllowSomePages = false;

                // Display the dialog. This returns true if the user presses the Print button.
                DialogResult result = d.ShowDialog(this);
                if (result == DialogResult.OK) {
                    using (Report report = new Report()) {
                        report.Amort = _amort;
                        report.PrinterSettings = d.PrinterSettings;
                        report.Print();
                    }
                }
            }
        }

        private void abountToolStripMenuItem_Click(object sender, EventArgs e) {

            using (AboutForm form = new AboutForm()) {
                form.ShowDialog();
            }
        }

        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e) {

            using (Report report = new Report()) {
                report.Amort = _amort;
                report.Preview();
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e) {

            this.Close();
        }
    }

    // NumericTextBox Class adapted from: http://msdn.microsoft.com/en-us/library/ms229644.aspx
    public class NumericTextBox : TextBox {

        private int _decPlaces = 0;
        private bool _exitOnEnter = false;

        // Restricts the entry of characters to digits (including hex), the negative sign,
        // the decimal point, and editing keystrokes (backspace).
        protected override void OnKeyPress(KeyPressEventArgs e) {

            base.OnKeyPress(e);

            System.Globalization.NumberFormatInfo numberFormatInfo = System.Globalization.CultureInfo.CurrentCulture.NumberFormat;
            string decimalSeparator = numberFormatInfo.NumberDecimalSeparator;
            string groupSeparator = numberFormatInfo.NumberGroupSeparator;
            string negativeSign = numberFormatInfo.NegativeSign;

            string keyInput = e.KeyChar.ToString();

            if (Char.IsDigit(e.KeyChar)) {
                // Digits are OK
            }
            else if (keyInput.Equals(decimalSeparator) ||
                keyInput.Equals(groupSeparator) ||
                keyInput.Equals(negativeSign)) {
                // Decimal separator is OK
            }
            else if (e.KeyChar == '\b') {
                // Backspace key is OK
            }
            else if (e.KeyChar == (char)Keys.Return) {
                if (_exitOnEnter) {
                    e.Handled = true;
                    SendKeys.Send("{TAB}");
                }
            }
            else {
                // Consume this invalid key and beep
                e.Handled = true;
            }
        }

        protected override void OnLeave(EventArgs e) {

            base.OnLeave(e);
            string format = "F" + _decPlaces;
            this.Text = this.DecimalValue.ToString(format);
        }

        public int IntValue {
            get {
                int tmp = 0;
                Int32.TryParse(this.Text, out tmp);
                return tmp;
            }
        }

        public decimal DecimalValue {
            get {
                decimal tmp = 0;
                Decimal.TryParse(this.Text, out tmp);
                return tmp;
            }
        }

        public int DecimalPlaces {
            get { return _decPlaces; }
            set {
                if (value >= 0) {
                    _decPlaces = value;
                }
                else {
                    throw new ArgumentException("Invalid Decimal Places");
                }
            }
        }

        public bool ExitOnEnter {
            get { return _exitOnEnter; }
            set { _exitOnEnter = value; }
        }
    }
}
