﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;

namespace Amortize {

    public class Report : PrintDocument {

        public Amort Amort { get; set; }
        int _margin;
        int _page;
        Font _font;
        int _item;

        public Report() {

            DocumentName = "Amortization Schedule";
        }

        public void Preview() {

            using (PrintPreviewDialog dlg = new PrintPreviewDialog()) {
                dlg.Document = this;
                ((Form)dlg).WindowState = FormWindowState.Maximized;
                dlg.ShowDialog();
            }
        }

        protected override void OnBeginPrint(PrintEventArgs e) {

            base.OnBeginPrint(e);

            // init values
            _font = new Font("Arial", 12, FontStyle.Bold);
            _page = 0;
            _item = 0;

            // 1 inch margin - scale is 1/100th inch for printers
            _margin = 100;
        }

        protected override void OnPrintPage(PrintPageEventArgs e) {

            base.OnPrintPage(e);

            StringFormat sf = new StringFormat();
            sf.Alignment = StringAlignment.Far;

            // page height
            int ph = DefaultPageSettings.PaperSize.Height;
            
            // line height
            float h = _font.GetHeight(e.Graphics);
            
            // print header
            float y = PrintHeader(e.Graphics);

            // print items
            while (_item < Amort.Schedule.Count) {

                // exit on page overflow
                if (y > (ph - _margin - (h * 2))) {
                    e.HasMorePages = true;
                    break;
                }

                AmortPayment a = Amort.Schedule[_item];
                e.Graphics.DrawString(a.Number.ToString(), _font, Brushes.Black, _margin, y);
                e.Graphics.DrawString(a.Date.ToString("MM/dd/yyyy"), _font, Brushes.Black, _margin + 100, y);
                Rectangle rect = new Rectangle((int)_margin + 200, (int)y, 100, (int)h);
                e.Graphics.DrawString(a.Interest.ToString("F2"), _font, Brushes.Black, rect, sf);
                rect = new Rectangle((int)_margin + 300, (int)y, 100, (int)h);
                e.Graphics.DrawString(a.Principle.ToString("F2"), _font, Brushes.Black, rect, sf);
                rect = new Rectangle((int)_margin + 400, (int)y, 100, (int)h);
                e.Graphics.DrawString(a.Balance.ToString("F2"), _font, Brushes.Black, rect, sf);
                y += h;

                _item++;
            }
        }

        private float PrintHeader(Graphics g) {

            _page++;

            // page height and width
            int ph = DefaultPageSettings.PaperSize.Height;
            int pw = DefaultPageSettings.PaperSize.Width;

            // header
            g.DrawString("Loan Amortization Schedule", _font, Brushes.Black, _margin, _margin);

            float y = _margin + 50;
            float h = _font.GetHeight(g);
            g.DrawString("Principle: " + Amort.Principle.ToString("F2"), _font, Brushes.Black, _margin, y);
            y += h;
            g.DrawString("Annual Interest Rate: " + Amort.Interest.ToString("F2"), _font, Brushes.Black, _margin, y);
            g.DrawString("Payment: " + Amort.Payment.ToString("F2"), _font, Brushes.Black, _margin + 300, y);
            y += h;
            g.DrawString("Number of Payments: " + Amort.Term.ToString(), _font, Brushes.Black, _margin, y);
            g.DrawString("Total Interest: " + Amort.TotalInterest.ToString("F2"), _font, Brushes.Black, _margin + 300, y);
            y += h * 2;

            StringFormat sf = new StringFormat();
            sf.Alignment = StringAlignment.Far;

            // column headers
            g.DrawString("Payment", _font, Brushes.Black, _margin, y);
            g.DrawString("Date", _font, Brushes.Black, _margin + 100, y);
            Rectangle rect = new Rectangle((int)_margin + 200, (int)y, 100, (int)h);
            g.DrawString("Interest", _font, Brushes.Black, rect, sf);
            rect = new Rectangle((int)_margin + 300, (int)y, 100, (int)h);
            g.DrawString("Principle", _font, Brushes.Black, rect, sf);
            rect = new Rectangle((int)_margin + 400, (int)y, 100, (int)h);
            g.DrawString("Balance", _font, Brushes.Black, rect, sf);

            y += h;

            // footer
            g.DrawLine(
                Pens.Black,
                _margin,
                ph - _margin,
                pw - _margin,
                ph - _margin);
            Font footFont = new Font("Arial", 10);
            g.DrawString(DateTime.Now.ToString(), footFont, Brushes.Black, _margin, ph - _margin);
            g.DrawString("Page: " + _page, footFont, Brushes.Black, pw - _margin - 60, ph - _margin);

            return y;
        }

    }
}
